@@ -1,12 +1,12 @@
 #############################################################################
 # Represents one node in a Graph::Easy
 #
-# (c) by Tels 2004-2006. Part of Graph::Easy
+# (c) by Tels 2004-2007. Part of Graph::Easy
 #############################################################################
 
 package Graph::Easy::Node;
 
-$VERSION = '0.33';
+$VERSION = '0.34';
 
 use Graph::Easy::Base;
 use Graph::Easy::Attributes;
@@ -15,8 +15,6 @@ use Graph::Easy::Attributes;
 my $att_aliases;
 
 use strict;
-use utf8;
-
 use constant isa_cell => 0;
 
 sub _init
@@ -406,7 +404,7 @@ sub _place
       }
     } # end handling of multi-celled node
 
-  $self->_update_boundaries();
+  $self->_update_boundaries($parent);
 
   1;					# did place us
   } 
@@ -1362,6 +1360,51 @@ sub is_anon
 #############################################################################
 # accessor methods
 
+sub _un_escape
+  {
+  # replace \N, \G, \T, \H and \E (depending on type)
+  # if $label is false, also replace \L with the label
+  my ($self, $txt, $do_label) = @_;
+ 
+  # for edges:
+  if (exists $self->{edge})
+    {
+    my $e = $self->{edge};
+    $txt =~ s/\\E/$e->{from}->{name}\->$e->{to}->{name}/g;
+    $txt =~ s/\\T/$e->{from}->{name}/g;
+    $txt =~ s/\\H/$e->{to}->{name}/g;
+    # \N for edges is the label of the edge
+    if ($txt =~ /\\N/)
+      {
+      my $l = $self->label();
+      $txt =~ s/\\N/$l/g;
+      }
+    }
+  else
+    {
+    # \N for nodes
+    $txt =~ s/\\N/$self->{name}/g;
+    }
+  # \L with the label
+  if ($txt =~ /\\L/ && $do_label)
+    {
+    my $l = $self->label();
+    $txt =~ s/\\L/$l/g;
+    }
+
+  # \G for edges and nodes
+  if ($txt =~ /\\G/)
+    {
+    my $g = '';
+    # the graph itself
+    $g = $self->attribute('title') unless ref($self->{graph});
+    # any nodes/edges/groups in it
+    $g = $self->{graph}->label() if ref($self->{graph});
+    $txt =~ s/\\G/$g/g;
+    }
+  $txt;
+  }
+
 sub title
   {
   # Returns a title of the node (or '', if none was set), which can be
@@ -1396,6 +1439,9 @@ sub title
       }
     $title = '' unless defined $title;
     }
+
+  $title = $self->_un_escape($title, 1) if !$_[0] && $title =~ /\\[EGHNTL]/;
+
   $title;
   }
 
@@ -1446,6 +1492,9 @@ sub label
         }
       }
     }
+
+  $label = $self->_un_escape($label) if !$_[0] && $label =~ /\\[EGHNT]/;
+
   $label;
   }
 
@@ -2081,12 +2130,10 @@ sub parent
 
 sub _update_boundaries
   {
-  my ($self) = @_;
+  my ($self, $parent) = @_;
 
   # XXX TODO: use current layout parent for recursive layouter:
-  #my $parent = $self->parent();
-
-  my $parent = $self->{graph};
+  $parent = $self->{graph};
 
   # cache max boundaries for A* algorithmn:
 
@@ -2288,10 +2335,15 @@ Return the name of the node.
 =head2 label()
 
 	my $label = $node->label();
+	my $label = $node->label(1);		# raw
 
 Return the label of the node. If no label was set, returns the C<name>
 of the node.
 
+If the optional parameter is true, then the label will returned 'raw',
+that is any potential escape of the form C<\N>, C<\E>, C<\G>, C<\T>
+or C<\H> will not be left alone and not be replaced.
+
 =head2 background()
 
 	my $bg = $node->background();
@@ -2309,18 +2361,30 @@ Returns the empty string if this object doesn't have a comment set.
 =head2 title()
 
 	my $title = $node->title();
+	my $title = $node->title(1);		# raw
 
 Returns a potential title that can be used for mouse-over effects.
 If no title was set (or autogenerated), will return an empty string.
 
+If the optional parameter is true, then the title will returned 'raw',
+that is any potential escape of the form C<\N>, C<\E>, C<\G>, C<\T>
+or C<\H> will not be left alone and not be replaced.
+
+
 =head2 link()
 
 	my $link = $node->link();
+	my $link = $node->link(1);		# raw
 
 Returns the URL, build from the C<linkbase> and C<link> (or C<autolink>)
 attributes.  If the node has no link associated with it, return an empty
 string.
 
+If the optional parameter is true, then the link will returned 'raw',
+that is any potential escape of the form C<\N>, C<\E>, C<\G>, C<\T>
+or C<\H> will not be left alone and not be replaced.
+
+
 =head2 dimensions()
 
 	my ($w,$h) = $node->dimensions();
